--******************************************************************************
--* Company:     FTDI                                                          *
--* Date:        23 Nov 2004   and uplifted 15 Sept 2010                                                *
--* Author:      Iain Maciver                                                  *
--*                                                                            *
--* Module Name: freq_cnt_top                                                  *
--*                                                                            *
--* Description: Calculates the frequency of an unknown input signal using     *
--*              Morph-IC clk as a reference                                   *
--*              50MHz Clock                                                   *
--*              Altera Cyclone-II device                                      *
--******************************************************************************

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY freq_cnt_top IS
  PORT (-- clk and rst signals
        clk50    : in std_logic;  	   -- 50MHz clock input
        FUnknown : in std_logic;       -- 12Mhz recovered clock input
        rst      : in std_logic;  	   -- Active high reset via AD7

        -- FT Signals
        BD       : inout std_logic_vector(7 downto 0);    -- FIFO Data Bus
        BC       : inout std_logic_vector(3 downto 0);    -- FIFO Control Bus
        siwub    : out   std_logic
        );
END freq_cnt_top;

ARCHITECTURE rtl OF freq_cnt_top IS

  COMPONENT freq_cnt_ctl
    PORT (
          clk50     : in std_logic;
          FUnknown  : in std_logic;
          rst       : in std_logic;
          FT245Din  : in std_logic_vector(7 downto 0);
          nRXF      : in std_logic;
          nTXE      : in std_logic;

          FT245Dout : out std_logic_vector(7 DOWNTO 0);
          FT245DEn  : out std_logic;
          nRD       : out std_logic;
          WR        : out std_logic
          );
  END COMPONENT;

  --
  signal BDin  : std_logic_vector(7 downto 0);
  signal BDout : std_logic_vector(7 downto 0);
  signal BDen  : std_logic_vector(7 downto 0);
  --
  signal BCin  : std_logic_vector(3 downto 0);
  signal BCout : std_logic_vector(3 downto 0);
  signal BCen  : std_logic_vector(3 downto 0);
  --
  signal nRD      : std_logic;   -- Read from FIFO
  signal WR       : std_logic;   -- Write to FIFO
  signal FT245DEn : std_logic;   -- enable BD output buffers
  --

BEGIN

--------------------------------------------------------------------------------
-- Create buffers for Bidi I/Os
--------------------------------------------------------------------------------
-- BD Bus buffers: Bi-directional and controlled by FT245DEn
  BDen(0) <= FT245DEn;
  BDen(1) <= FT245DEn;
  BDen(2) <= FT245DEn;
  BDen(3) <= FT245DEn;
  BDen(4) <= FT245DEn;
  BDen(5) <= FT245DEn;
  BDen(6) <= FT245DEn;
  BDen(7) <= FT245DEn;
  
  bfBD0 : process (BD(0), BDout(0), BDen(0))
  begin
    if (BDen(0)='1') then
      BD(0) <= BDout(0);
    else
      BD(0) <= 'Z';
    end if;
    BDin(0) <= BD(0);
  end process bfBD0;
  
  bfBD1 : process (BD(1), BDout(1), BDen(1))
  begin
    if (BDen(1)='1') then
      BD(1) <= BDout(1);
    else
      BD(1) <= 'Z';
    end if;
    BDin(1) <= BD(1);
  end process bfBD1;
  
  bfBD2 : process (BD(2), BDout(2), BDen(2))
  begin
    if (BDen(2)='1') then
      BD(2) <= BDout(2);
    else
      BD(2) <= 'Z';
    end if;
    BDin(2) <= BD(2);
  end process bfBD2;
  
  bfBD3 : process (BD(3), BDout(3), BDen(3))
  begin
    if (BDen(3)='1') then
      BD(3) <= BDout(3);
    else
      BD(3) <= 'Z';
    end if;
    BDin(3) <= BD(3);
  end process bfBD3;
  
  bfBD4 : process (BD(4), BDout(4), BDen(4))
  begin
    if (BDen(4)='1') then
      BD(4) <= BDout(4);
    else
      BD(4) <= 'Z';
    end if;
    BDin(4) <= BD(4);
  end process bfBD4;
  
  bfBD5 : process (BD(5), BDout(5), BDen(5))
  begin
    if (BDen(5)='1') then
      BD(5) <= BDout(5);
    else
      BD(5) <= 'Z';
    end if;
    BDin(5) <= BD(5);
  end process bfBD5;
  
  bfBD6 : process (BD(6), BDout(6), BDen(6))
  begin
    if (BDen(6)='1') then
      BD(6) <= BDout(6);
    else
      BD(6) <= 'Z';
    end if;
    BDin(6) <= BD(6);
  end process bfBD6;
  
  bfBD7 : process (BD(7), BDout(7), BDen(7))
  begin
    if (BDen(7)='1') then
      BD(7) <= BDout(7);
    else
      BD(7) <= 'Z';
    end if;
    BDin(7) <= BD(7);
  end process bfBD7;

-- BC Bus buffers: BC (0) and (1) are inputs RXFn and TXEn
--                 BC (2) and (3) are outputs nRD and nWR
  BCEn(0) <= '0';  -- input
  BCEn(1) <= '0';  -- input
  BCEn(2) <= '1';  -- output
  BCEn(3) <= '1';  -- output
  
  BCout(0) <= '1';
  BCout(1) <= '1';
  BCout(2) <= nRD; -- Read from channel B FIFO when nRD is low
  BCout(3) <= WR;  -- Write to channel B FIFO
  
  bfBC0 : process (BC(0), BCout(0), BCen(0))
  begin
    if (BCen(0)='1') then
      BC(0) <= BCout(0);
    else
      BC(0) <= 'Z';
    end if;
    BCin(0) <= BC(0);
  end process bfBC0;
  
  bfBC1 : process (BC(1), BCout(1), BCen(1))
  begin
    if (BCen(1)='1') then
      BC(1) <= BCout(1);
    else
      BC(1) <= 'Z';
    end if;
    BCin(1) <= BC(1);
  end process bfBC1;
  
  bfBC2 : process (BC(2), BCout(2), BCen(2))
  begin
    if (BCen(2)='1') then
      BC(2) <= BCout(2);
    else
      BC(2) <= 'Z';
    end if;
    BCin(2) <= BC(2);
  end process bfBC2;
  
  bfBC3 : process (BC(3), BCout(3), BCen(3))
  begin
    if (BCen(3)='1') then
      BC(3) <= BCout(3);
    else
      BC(3) <= 'Z';
    end if;
    BCin(3) <= BC(3);
  end process bfBC3;
  
--------------------------------------------------------------------------------
-- Wrapper 
--------------------------------------------------------------------------------
  i_freq_ctl : freq_cnt_ctl PORT MAP(
    clk50     => clk50,
    FUnknown  => FUnknown,
    rst       => rst,
    FT245Din  => BDin,
    nRXF      => BC(0),
    nTXE      => BC(1),
    
    FT245Dout => BDout,
    FT245DEn  => FT245DEn,
    nRD       => nRD,
    WR        => WR
    );

  siwub <= '1';

END rtl;

